@@ -25,6 +25,7 @@ from cmdstanpy.utils import (
     rload,
     parse_rdump_value,
     jsondump,
+    parse_var_dims,
 )
 from cmdstanpy.model import CmdStanModel
 
@@ -147,48 +148,81 @@ class CmdStanPathTest(unittest.TestCase):
                 pass
 
     def test_jsondump(self):
+        def cmp(d1, d2):
+            self.assertEqual(d1.keys(), d2.keys())
+            for k in d1:
+                data_1 = d1[k]
+                data_2 = d2[k]
+                if isinstance(data_2, np.ndarray):
+                    data_2 = data_2.tolist()
+                self.assertEqual(data_1, data_2)
+
         dict_list = {'a': [1.0, 2.0, 3.0]}
         file_list = os.path.join(_TMPDIR, 'list.json')
         jsondump(file_list, dict_list)
         with open(file_list) as fd:
-            self.assertEqual(json.load(fd), dict_list)
+            cmp(json.load(fd), dict_list)
 
         dict_vec = {'a': np.repeat(3, 4)}
         file_vec = os.path.join(_TMPDIR, 'vec.json')
         jsondump(file_vec, dict_vec)
         with open(file_vec) as fd:
-            self.assertEqual(json.load(fd), dict_vec)
+            cmp(json.load(fd), dict_vec)
 
         dict_zero_vec = {'a': []}
         file_zero_vec = os.path.join(_TMPDIR, 'empty_vec.json')
         jsondump(file_zero_vec, dict_zero_vec)
         with open(file_zero_vec) as fd:
-            self.assertEqual(json.load(fd), dict_zero_vec)
+            cmp(json.load(fd), dict_zero_vec)
 
         dict_zero_matrix = {'a': [[], [], []]}
         file_zero_matrix = os.path.join(_TMPDIR, 'empty_matrix.json')
         jsondump(file_zero_matrix, dict_zero_matrix)
         with open(file_zero_matrix) as fd:
-            self.assertEqual(json.load(fd), dict_zero_matrix)
+            cmp(json.load(fd), dict_zero_matrix)
 
         arr = np.zeros(shape=(5, 0))
         dict_zero_matrix = {'a': arr}
         file_zero_matrix = os.path.join(_TMPDIR, 'empty_matrix.json')
         jsondump(file_zero_matrix, dict_zero_matrix)
         with open(file_zero_matrix) as fd:
-            self.assertEqual(json.load(fd), dict_zero_matrix)
+            cmp(json.load(fd), dict_zero_matrix)
 
 
 class ReadStanCsvTest(unittest.TestCase):
     def test_check_sampler_csv_1(self):
         csv_good = os.path.join(DATAFILES_PATH, 'bernoulli_output_1.csv')
-        dict = check_sampler_csv(csv_good)
+        dict = check_sampler_csv(
+            path=csv_good,
+            is_fixed_param=False,
+            iter_warmup=100,
+            iter_sampling=10,
+            thin=1,
+        )
         self.assertEqual('bernoulli_model', dict['model'])
         self.assertEqual(10, dict['num_samples'])
         self.assertFalse('save_warmup' in dict)
-        self.assertEqual(10, dict['draws'])
+        self.assertEqual(10, dict['draws_sampling'])
         self.assertEqual(8, len(dict['column_names']))
 
+        with self.assertRaisesRegex(
+            ValueError, 'config error, expected thin = 2'
+        ):
+            check_sampler_csv(
+                path=csv_good, iter_warmup=100, iter_sampling=20, thin=2
+            )
+        with self.assertRaisesRegex(
+            ValueError, 'config error, expected save_warmup'
+        ):
+            check_sampler_csv(
+                path=csv_good,
+                iter_warmup=100,
+                iter_sampling=10,
+                save_warmup=True,
+            )
+        with self.assertRaisesRegex(ValueError, 'expected 1000 draws'):
+            check_sampler_csv(path=csv_good, iter_warmup=100)
+
     def test_check_sampler_csv_2(self):
         csv_bad = os.path.join(DATAFILES_PATH, 'no_such_file.csv')
         with self.assertRaises(Exception):
@@ -236,7 +270,7 @@ class ReadStanCsvTest(unittest.TestCase):
         bern_fit = bern_model.sample(
             data=jdata,
             chains=1,
-            cores=1,
+            parallel_chains=1,
             seed=12345,
             iter_sampling=490,
             iter_warmup=490,
@@ -245,14 +279,36 @@ class ReadStanCsvTest(unittest.TestCase):
             adapt_delta=0.98,
         )
         csv_file = bern_fit.runset.csv_files[0]
-        dict = check_sampler_csv(csv_file)
+        dict = check_sampler_csv(
+            path=csv_file,
+            is_fixed_param=False,
+            iter_sampling=490,
+            iter_warmup=490,
+            thin=7,
+        )
         self.assertEqual(dict['num_samples'], 490)
         self.assertEqual(dict['thin'], 7)
-        self.assertEqual(dict['draws'], 70)
+        self.assertEqual(dict['draws_sampling'], 70)
         self.assertEqual(dict['seed'], 12345)
         self.assertEqual(dict['max_depth'], 11)
         self.assertEqual(dict['delta'], 0.98)
 
+        with self.assertRaisesRegex(ValueError, 'config error'):
+            check_sampler_csv(
+                path=csv_file,
+                is_fixed_param=False,
+                iter_sampling=490,
+                iter_warmup=490,
+                thin=9,
+            )
+        with self.assertRaisesRegex(ValueError, 'expected 490 draws, found 70'):
+            check_sampler_csv(
+                path=csv_file,
+                is_fixed_param=False,
+                iter_sampling=490,
+                iter_warmup=490,
+            )
+
 
 class ReadMetricTest(unittest.TestCase):
     def test_metric_json_vec(self):
@@ -434,5 +490,72 @@ class RloadTest(unittest.TestCase):
         self.assertEqual(v_struct3[6, 1], 15)
 
 
+class ParseVarDimsTest(unittest.TestCase):
+    def test_parse_empty(self):
+        x = []
+        vars_dict = parse_var_dims(x)
+        self.assertEqual(len(vars_dict), 0)
+
+    def test_parse_missing(self):
+        with self.assertRaises(ValueError):
+            parse_var_dims(None)
+
+    def test_parse_scalar(self):
+        x = ['foo']
+        vars_dict = parse_var_dims(x)
+        self.assertEqual(len(vars_dict), 1)
+        self.assertEqual(vars_dict['foo'], 1)
+
+    def test_parse_scalars(self):
+        x = ['foo', 'foo1']
+        vars_dict = parse_var_dims(x)
+        self.assertEqual(len(vars_dict), 2)
+        self.assertEqual(vars_dict['foo'], 1)
+        self.assertEqual(vars_dict['foo1'], 1)
+
+    def test_parse_scalar_vec_scalar(self):
+        x = [
+            'foo',
+            'phi.1',
+            'phi.2',
+            'phi.3',
+            'phi.4',
+            'phi.5',
+            'phi.6',
+            'phi.7',
+            'phi.8',
+            'phi.9',
+            'phi.10',
+            'bar',
+        ]
+        vars_dict = parse_var_dims(x)
+        self.assertEqual(len(vars_dict), 3)
+        self.assertEqual(vars_dict['foo'], 1)
+        self.assertEqual(vars_dict['phi'], (10,))
+        self.assertEqual(vars_dict['bar'], 1)
+
+    def test_parse_scalar_matrix_vec(self):
+        x = [
+            'foo',
+            'phi.1.1',
+            'phi.1.2',
+            'phi.1.3',
+            'phi.1.4',
+            'phi.1.5',
+            'phi.2.1',
+            'phi.2.2',
+            'phi.2.3',
+            'phi.2.4',
+            'phi.2.5',
+            'bar.1',
+            'bar.2',
+        ]
+        vars_dict = parse_var_dims(x)
+        self.assertEqual(len(vars_dict), 3)
+        self.assertEqual(vars_dict['foo'], 1)
+        self.assertEqual(vars_dict['phi'], (2, 5))
+        self.assertEqual(vars_dict['bar'], (2,))
+
+
 if __name__ == '__main__':
     unittest.main()
